
/**************************************************************
 ***************** NBTV Receiver ****************************** 
 *********** Receives and displays NBTV video ***************** 
 ************** in color or monochrome mode *******************
 ******************* at 32 or 64 lines ************************
 ******************** at 12.5 or 15 fps ************************
 **************************************************************/
 

#include <SSD_13XX.h>
#include <SPI.h>
#include <DueTimer.h>

#define __CS1  22 
#define __DC  24  
#define __RS  26

#define fpsButton 36
#define colorBWbutton 38
#define nLinesButton 40

#define BLUE_LINE 1
#define RED_LINE 0

#define debug false

SSD_13XX tft = SSD_13XX(__CS1, __DC);

bool v_sync_detected = false;
bool color_line = RED_LINE;
uint8_t red_buffer[100];
uint8_t blue_buffer[100];
uint16_t line_buffer[100];
uint16_t disp_line_buffer[100];
volatile uint8_t num_lines = 64;
volatile uint8_t pic_pixels = 86;

volatile uint8_t pix_nbr = 0;

volatile bool waiting_for_sync = false;
volatile uint8_t sync_threshold = 5;
uint8_t video_threshold= 10;
uint16_t sync_count = 0;
volatile uint8_t line_zoom = 2;

char * c_mode[6] = {"Monochrome","RGB color","RG color","RC color","OG Color","RGB1 Color"};
char * fps_mode[2] = {"12.5 fps","15.0 fps"};
char * lines_mode[2] = {"32 lines","64 lines"};

float fps[2] = {12.5,15.0};
uint8_t nLines[2] ={64,64};
uint8_t lZoom[2] = {2,1};

uint8_t pix_offset = 5;
uint16_t pixel_duration ; 
bool autodetect = false;

struct parms
{
uint8_t fps_index = 1;
uint8_t n_lines_index = 1;
uint8_t color_mode =1;
};
struct parms p;


boolean mode_detected=false;

//==================== Switch between 32 or 64 lines ====================================//

void check_num_lines()
{
   if(digitalRead(nLinesButton) == LOW)
  {
   delay(100);
    if(digitalRead(nLinesButton) == LOW)
    {
      p.n_lines_index++;
      if(p.n_lines_index==2)
      {
        p.n_lines_index = 0;
      }
      tft.fillScreen(BLACK);
      calculate_pix_rate();   
      display_parm_change(1);
    }
  }
}

//==================== Switch between 12.5 or 15 fps  ====================================//

void check_fr()
{
   if(digitalRead(fpsButton) == LOW)
  {
   delay(100);
    if(digitalRead(fpsButton) == LOW)
    {
      p.fps_index++;
      if(p.fps_index==2)
      {
        p.fps_index = 0;
      }
      calculate_pix_rate();
      
      display_parm_change(0); 
//      autodetect=true;
    }
  }
}

//==================== Switch color mode ====================================//

void check_color()
{
   if(digitalRead(colorBWbutton) == LOW)
  {
    delay(100);
    if(digitalRead(colorBWbutton) == LOW)
    {
      p.color_mode++;
      if(p.color_mode == 6)
        p.color_mode = 0;
     display_parm_change(2); 
    }
  }
}


//==================== Timer interrupt routine ==================================//

void timer1_int()
{
  if(waiting_for_sync)
    return;
    int ch0 = get_adcA0();
    int ch1 = get_adcA1();

     ch0=ch0>>2;
     ch1=ch1>>3;
     if(p.color_mode==0)
     {
      line_buffer[pix_nbr] =  ch0<<5 | (ch0>>1)<<11 | (ch0>>1);   
     }
     else
     {
     if(p.color_mode==2)
     {
      line_buffer[pix_nbr] = ch0<<5 |ch1<< 11 | 3 ;
     }
     else if(p.color_mode==3)
     {
      line_buffer[pix_nbr] = ch0<<5 |ch1<< 11 | (ch0>>1) ;
     }
     else if(p.color_mode == 4)
     {
      line_buffer[pix_nbr] = ((ch0>>1)+ch1)<<5 |ch1<< 11 | (ch0>>1) ;
     }
     else if(p.color_mode == 5)
     {
      if(pix_nbr<pic_pixels/2 )
      {
       
        red_buffer[(pix_nbr)*2] = ch1;
        red_buffer[(pix_nbr)*2+1] = ch1;
       
        
      }
      else if(pix_nbr>pic_pixels/2 + 4)
      {
        blue_buffer[(pix_nbr-4-(pic_pixels/2))*2] = ch1;
        blue_buffer[(pix_nbr-4-(pic_pixels/2))*2+1] = ch1;
      }
      line_buffer[pix_nbr] =ch0<<5 | (red_buffer[pix_nbr]<<11) | blue_buffer[pix_nbr];  
     }
     else
     {
      if(color_line == BLUE_LINE)  // odd lines are blue
      {
       blue_buffer[pix_nbr] = ch1;           
      }
      else  // even lines are red
      { 
       red_buffer[pix_nbr] = ch1;
      }
      line_buffer[pix_nbr] =ch0<<5 | (red_buffer[pix_nbr]<<11) | blue_buffer[pix_nbr];          
     }   
  }
  pix_nbr++;
  if(pix_nbr == pic_pixels+6)
  {
    color_line =! color_line;
     waiting_for_sync= true;
    memcpy(disp_line_buffer,line_buffer,pic_pixels*sizeof(uint16_t));
 
  }
}

//==================== Calculate pixel rate ====================================//

void calculate_pix_rate()
{
  Timer3.stop();
  float pixRate;
  num_lines = nLines[p.n_lines_index];
  line_zoom = lZoom[p.n_lines_index];  
  float sync_length = pic_pixels/12;
  int numPixPerFrame = (num_lines/line_zoom)*(pic_pixels+sync_length );
  pixRate = (1000000.0/fps[p.fps_index])/numPixPerFrame; 
  pixel_duration = (uint8_t) pixRate-1 ; 
  pix_nbr=0;
 
}

//=================== ON SCREEN DISPLAY WHEN PARAMTER CHANGED ================//
 
void display_parm_change(uint8_t parm_change)
{
  Timer3.stop();
  waiting_for_sync= false;
 
  pix_nbr=0;
  tft.setTextScale(1);
  tft.setTextColor(YELLOW);
    
  
  switch(parm_change)
  {
    case 0:
    tft. setCursor(CENTER,55);
    tft.println(fps_mode[p.fps_index]);
    break;
    case 1:
    tft. setCursor(CENTER,55);
    tft.println(lines_mode[p.n_lines_index]);
    break;
    case 2:
    tft. setCursor(CENTER,55);
    tft.println(c_mode[p.color_mode]);
    break;
    case 3:
    tft. setCursor(5,55);
    tft.print(fps_mode[p.fps_index]);
    tft.print("  ");
    tft.println(lines_mode[p.n_lines_index]);
    break;
    case 4:
    tft. setCursor(2,55);
    tft.println("Autodetect failed");
    break;
    default:
    break;
  }
  delay(1000);  
  tft.fillScreen(BLACK);
  Timer3.start(pixel_duration);
}

//======================== INITIAL SCREEN AT BOOT UP ========================//

void display_mode()
{
  Timer3.stop();
  waiting_for_sync= false;
  tft.fillScreen(BLUE);
  tft.setTextColor(YELLOW);
  tft.setTextScale(1);
  tft. setCursor(CENTER,5);
  tft.println("NBTV Receiver");
  tft. setCursor(CENTER,15);
  tft.println("KD6CJI, 2018");
  tft.println(" ");
  tft. setCursor(CENTER,30);
  tft.println(fps_mode[p.fps_index]);
  tft. setCursor(CENTER,40);
   tft.println(lines_mode[p.n_lines_index]);
  tft. setCursor(CENTER,50);
  tft.println(c_mode[p.color_mode]);  
  delay(3000);
  tft.fillScreen(BLACK);
  Timer3.start(pixel_duration);
}

//================================ SETUP ====================================//

void setup() 
{
  if(debug)
  {
  Serial.begin(38400);
  long unsigned debug_start = millis ();
  while (!Serial && ((millis () - debug_start) <= 5000)) ;
  }  
  ADC->ADC_MR |= 0x80;  //set free running mode on ADC
  ADC->ADC_CHER = 0xC0; //enable ADC on pin A0 and A1  
  pinMode(fpsButton, INPUT_PULLUP);
  pinMode(colorBWbutton, INPUT_PULLUP);
  pinMode(nLinesButton,INPUT_PULLUP);
  pinMode(__RS,OUTPUT);
  digitalWrite(__RS,HIGH);
  Timer3.attachInterrupt(timer1_int);
  tft.begin();
  tft.setBrightness(15);
  tft.setTextColor(YELLOW);
  calculate_pix_rate();
  display_mode();
 tft.setArea(pix_offset,0,pic_pixels+pix_offset-1,num_lines-2);
}

//==================== ADC on pin A0 ====================================//

uint8_t get_adcA0()
{
   while((ADC->ADC_ISR & 0x80)==0); // wait for conversion
   return (ADC->ADC_CDR[7]>>4);
}

//==================== ADC on pin A1 ====================================//

uint8_t get_adcA1()
{
   while((ADC->ADC_ISR & 0x40)==0); // wait for conversion
   return (ADC->ADC_CDR[6]>>4);
}

//==================== Display routine ====================================//

void move_to_display()
{
  uint8_t x,y;
  
   for(y=0;y<line_zoom;y++)
   {
    for(x=0;x<pic_pixels;x++)
    {
       tft.pushColor(disp_line_buffer[x]);       
     }     
   }
}

//==================== Processing loop ====================================//

void loop() 
{

  check_color();
  check_fr();
  check_num_lines();

  if(waiting_for_sync)
  {
  Timer3.stop();
  waiting_for_sync = false;
   pix_nbr=0;
  // delayMicroseconds(pixel_duration*7);
   while(get_adcA0() > 0 || get_adcA1() > 0);//sync_threshold);
   
   if(p.color_mode == 5)
     Timer3.start(pixel_duration);
//delayMicroseconds(pixel_duration*4);
   while(get_adcA0() <= 0 || get_adcA1() <= 0)//sync_threshold)
   {
    sync_count++;
   }
  if(p.color_mode != 5)
   Timer3.start(pixel_duration);
   
   if(sync_count > 100 )
   { 
   
    
    v_sync_detected = true;
    color_line=RED_LINE; 
   }
    
    sync_count=0; 
  
    move_to_display();
   if(v_sync_detected)
   { 
    tft.setArea(pix_offset,0,pic_pixels+pix_offset-1,num_lines-2); 
     v_sync_detected= false;
   }
  
  } 
}

//==================== END ====================================//
